// FlashQuizDlg.cpp : implementation file
//
// FlashQuiz v1.0
// Copyright  2000 Ziff Davis Media, Inc.
// Written by Gregory A. Wolking.

#include "stdafx.h"
#include "FlashQuiz.h"
#include "FlashQuizDlg.h"
#include "PwdDlg.h"
#include "ParmDlg.h"
#include "MultChoice.h"
#include "FlashCard.h"
#include "QuizEditor.h"
#include "LogScore.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CFlashQuizApp theApp;
extern CFlashQuizDoc theDoc;

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFlashQuizDlg dialog

CFlashQuizDlg::CFlashQuizDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CFlashQuizDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CFlashQuizDlg)
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_pQuestionDlg = NULL;
}

void CFlashQuizDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFlashQuizDlg)
	DDX_Control(pDX, IDC_cmdTakeQuiz, m_cmdTakeQuiz);
	DDX_Control(pDX, IDC_cmdEdit, m_cmdEdit);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CFlashQuizDlg, CDialog)
	//{{AFX_MSG_MAP(CFlashQuizDlg)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_cmdNew, OncmdNew)
	ON_BN_CLICKED(IDC_cmdAbout, OncmdAbout)
	ON_BN_CLICKED(IDC_cmdEdit, OncmdEdit)
	ON_BN_CLICKED(IDC_cmdLoad, OncmdLoad)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_cmdTakeQuiz, OncmdTakeQuiz)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
	ON_MESSAGE(MY_TAKE_QUIZ, OnTakeQuiz)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFlashQuizDlg message handlers

// Overrides for intrinsic CDialog handlers.
BOOL CFlashQuizDlg::OnInitDialog()
{
	CFont* pFont;

	CDialog::OnInitDialog();
	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	Set_Doc_Title();
	if (!theDoc.m_bCustomFont)
	{
		pFont = GetFont();
		pFont->GetLogFont(&theDoc.m_fontCustom);
	}
	if (!theApp.m_strFileName.IsEmpty())
		PostMessage(MY_TAKE_QUIZ, 0, 0);
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CFlashQuizDlg::OnDestroy()
{
	WinHelp(0L, HELP_QUIT);
	CDialog::OnDestroy();
}

void CFlashQuizDlg::OnOK() 
{
	if (theDoc.Do_Query_Save(this))
		CDialog::OnOK();
}

// Empty override to disable the {Esc} key.
void CFlashQuizDlg::OnCancel()
{
}

// Base class OnClose() handler routes through OnCancel(),
// so call EndDialog() directly instead.
void CFlashQuizDlg::OnClose() 
{
	if (theDoc.Do_Query_Save(this))
		EndDialog(IDOK);
}

//////////////////////////////////
// Button event handlers.
void CFlashQuizDlg::OncmdNew() 
{
	CPwdDlg		dlgPwd;
	CQuizEditor dlgEditor;
	CParmDlg	dlgParms;

	if (!theDoc.Do_Query_Save(this))
		return;
	theDoc.DeleteContents();
	dlgPwd.m_intMode = 1;
	if (dlgPwd.DoModal() != IDOK)
		return;
	dlgParms.m_bInitialParms = TRUE;
	if (dlgParms.DoModal() != IDOK)
		return;
	dlgEditor.DoModal();
	m_cmdTakeQuiz.EnableWindow(theDoc.m_intQuestionCount != 0);
	m_cmdEdit.EnableWindow(!theDoc.m_strPassword.IsEmpty());
	Set_Doc_Title();
}

void CFlashQuizDlg::OncmdAbout() 
{
	CAboutDlg dlgAbout;
	dlgAbout.DoModal();
}

void CFlashQuizDlg::OncmdEdit() 
{
	int			iResult;
	CPwdDlg		dlgPwd;
	CQuizEditor dlgEditor(this);
	CString		strTemp, strText;

	dlgPwd.m_intMode = 0;
	iResult = dlgPwd.DoModal();
	switch (iResult)
	{
	case IDOK:
		break;
	case IDABORT:
		MessageBox("Access Denied!", "FlashQuiz", MB_ICONSTOP);
	default:
		return;
	}
	dlgEditor.DoModal();
	m_cmdTakeQuiz.EnableWindow(theDoc.m_intQuestionCount != 0);
	Set_Doc_Title();
}

void CFlashQuizDlg::OncmdLoad() 
{
	int			iResult;
	CString		strFileName, strTemp, strTitle;

	if (!theDoc.Do_Query_Save(this))
		return;
	m_cmdEdit.EnableWindow(FALSE);
	theDoc.DeleteContents();
	CFileDialog dlg(TRUE, ".fqz", NULL, OFN_PATHMUSTEXIST, "FlashQuiz Files (*.fqz)|*.fqz||", this);
	iResult = dlg.DoModal();
	if (iResult == IDOK)
	{
		strFileName = dlg.GetPathName();
		Load_Document(strFileName);
	}
	m_cmdTakeQuiz.EnableWindow(theDoc.m_intQuestionCount != 0);
	m_cmdEdit.EnableWindow(!theDoc.m_strPassword.IsEmpty());
}

void CFlashQuizDlg::OncmdTakeQuiz() 
{
	int			i, j, iMaxScore, iAnswered;
	int			*iIndexArray = NULL;
	BOOL		bCompleted = TRUE, bQuizType;
	CString		strTemp;
	CMultChoice	dlgMultChoice;
	CFlashCard	dlgFlashCard;
	CQuestion*	pQuestion;

	bQuizType = (theDoc.m_intQuizType == 0);
	theDoc.Reset_Quiz();
	theDoc.m_intPossibleScore = 0;
	theDoc.m_intTotalScore = 0;
	// Set remaining quiz time in seconds.
	theDoc.m_intTimeLeft = theDoc.m_intTimeLimit * 60;
	// Create array for question order.
	iIndexArray = new int[theDoc.m_intQuestionCount];
	// Report error if allocation failed.
	if (iIndexArray == NULL)
	{
		MessageBox("Memory allocation error", "FlashQuiz", MB_ICONINFORMATION);
		return;
	}
	else	// Allocation succeeded:
	{
		if (theDoc.m_bRandomQ)	// Random flag set?
		{
			// Build a random list.
			if (!theApp.Randomize_List(iIndexArray, theDoc.m_intQuestionCount))
			{
				MessageBox("Memory allocation error", "FlashQuiz", MB_ICONINFORMATION);
				delete [] iIndexArray;
				return;
			}
		}
		else		// Random flag not set, so
		{
			// present questions in "raw" order.
			for (i = 0; i < theDoc.m_intQuestionCount; i++)
				iIndexArray[i] = i;
		}
	}
	// Set dialog pointer for the OnTimer() handler.
	if (bQuizType)
		m_pQuestionDlg = (CDialog*) &dlgMultChoice;
	else
		m_pQuestionDlg = (CDialog*) &dlgFlashCard;
	// Create a timer to fire at one-second intervals.
	m_intMyTimer = SetTimer(1, 1000, NULL);
	// Report error if no timers were available.
	if (m_intMyTimer == 0)
	{
		MessageBox("SetTimer() API call failed.", "FlashQuiz", MB_ICONINFORMATION);
		delete [] iIndexArray;
		return;
	}
	iAnswered = 0;	// Initialize count of questions answered.
	// Loop through the index array.
	for (i = 0; i < theDoc.m_intQuestionCount; i++)
	{
		// Retrieve pointer to the current question.
		pQuestion = theDoc.Get_Question(iIndexArray[i]);
		// Launch the appropriate dialog for the current quiz type.
		if (bQuizType)
		{
			dlgMultChoice.m_intQuestion = i + 1;	// Set question number for display.
			dlgMultChoice.m_pQuestion = pQuestion;	// Give dialog a pointer to the question.
			// Launch the dialog. If result is not IDOK, then the user either
			// cancelled the quiz or, if timed, the timer expired.
			if (dlgMultChoice.DoModal() != IDOK)
			{
				bCompleted = FALSE;
				break;
			}
		}
		else
		{
			dlgFlashCard.m_intQuestion = i + 1;
			dlgFlashCard.m_pQuestion = pQuestion;
			if (dlgFlashCard.DoModal() != IDOK)
			{
				bCompleted = FALSE;
				break;
			}
		}
		// If we got this far, the user actually answered the question.
		// Update possible score from the current question.
		theDoc.m_intPossibleScore += pQuestion->m_intScore;
		++iAnswered;	// Increment count.
	}
	// Dialog window no longer exists: NULL its pointer in case the
	// timer fires before we can shut it down.
	m_pQuestionDlg = NULL;
	if (m_intMyTimer != 0)			// If the timer exists,
	{
		KillTimer(m_intMyTimer);	// kill it.
		m_intMyTimer = 0;
	}
	// If quiz was timed and the timer expired,
	if (theDoc.m_bTimeLimit && theDoc.m_intTimeLeft < 0)
	{
		if (iAnswered == 0)		// No questions were answered.
		{
			MessageBox("You answered no questions at all.", "Time Expired", MB_ICONINFORMATION);
		}
		else				// At least one question was answered.
		{
			// Calculate maximum possible score for the entire quiz.
			iMaxScore = theDoc.m_intPossibleScore;
			for (j = i; j < theDoc.m_intQuestionCount; j++)
			{
				pQuestion = theDoc.Get_Question(iIndexArray[j]);
				iMaxScore += pQuestion->m_intScore;
			}
			// Display score.
			strTemp.Format("For the %u question(s) you answered, you scored %u of %u points (%u%%).\n"
				"For the entire quiz, you scored %u of %u points (%u%%).",
				iAnswered,
				theDoc.m_intTotalScore,
				theDoc.m_intPossibleScore,
				(100 * theDoc.m_intTotalScore) / theDoc.m_intPossibleScore,
				theDoc.m_intTotalScore,
				iMaxScore,
				(100 * theDoc.m_intTotalScore) / iMaxScore);
			MessageBox(strTemp, "Time Expired!", MB_ICONINFORMATION);
			Do_Log();
		}
	}
	if (bCompleted)		// If entire quiz was completed,
	{
		// Calculate and display the score.
		strTemp.Format("Final score: %u of %u points (%u%%)",
			theDoc.m_intTotalScore,
			theDoc.m_intPossibleScore,
			(100 * theDoc.m_intTotalScore) / theDoc.m_intPossibleScore);
		MessageBox(strTemp);
		Do_Log();
	}
	delete [] iIndexArray;	// Delete the question index array.
}

//////////////////////////////////////
// App-specific message handlers.

// This is the "master timer" handler. The timer is created by OncmdTakeQuiz,
// which also sets the pointer for the question dialog.
// This function decrements the total quiz time (if appropriate), then
// notifies the appropriate dialog that the timer has fired.
// It is then the dialog's responsibility to update its own timeout for
// the current question (if appropriate), as well as checking to see if the
// time for the current quiz has expired.
void CFlashQuizDlg::OnTimer(UINT nIDEvent) 
{
	if (theDoc.m_bTimeLimit)			// If current quiz is timed,
		--theDoc.m_intTimeLeft;			// Decrement time left.
	// Make sure dialog pointer is set, and
	if (m_pQuestionDlg != NULL)
		// that the dialog window actually exists, then
		if (m_pQuestionDlg->m_hWnd != NULL)
			// notify the dialog that the timer fired.
			m_pQuestionDlg->PostMessage(MY_UPDATE_TIMER, 0, 0);
	CDialog::OnTimer(nIDEvent);
}

// Handles message posted when filename is specified on the command line.
LONG CFlashQuizDlg::OnTakeQuiz(WPARAM, LPARAM)
{
	if (Load_Document(theApp.m_strFileName))	// Try to load the file.
	{
		if (theApp.m_bEditFile)	// If "/EDIT" parm specified,
			OncmdEdit();		// Edit the quiz.
		else					// otherwise,
			OncmdTakeQuiz();	// present the quiz.
	}
	if (!theApp.m_bEditFile)	// If presenting the quiz, quit when quiz completes.
		EndDialog(IDOK);
	return 0L;
}

///////////////////////////////////
// Helper functions.

// Sets window caption based on current quiz filename.
void CFlashQuizDlg::Set_Doc_Title()
{
	int		iPos;
	CString strTemp,
			strText("FlashQuiz - <New Quiz>");	// Default caption.
	CFile	file;

	if (!theDoc.m_strFileName.IsEmpty())		// Is filename assigned?
	{
		// Yes -- extract the filename itself.
		iPos = theDoc.m_strFileName.ReverseFind('\\');
		if (iPos >= 0)
		{
			strTemp = theDoc.m_strFileName.Mid(iPos + 1);	// Format the string.
			strText.Format("FlashQuiz - %s", strTemp);
		}
	}
	SetWindowText(strText);	// Set the window caption.
}

// Loads the specified quiz file.
BOOL CFlashQuizDlg::Load_Document(LPCTSTR pszFileName)
{
	CFile		*pFile = NULL;
	CArchive	*pArchive = NULL;
	CString		strTemp, strTitle;

	// First, try to open the file.
	try
	{
		pFile = new CFile(pszFileName, CFile::modeRead | CFile::shareDenyWrite);
	}
	catch (CFileException *e)
	{
		e->Delete();
		strTemp.Format("Could not open\n%s", pszFileName);
		MessageBox(strTemp, "FlashQuiz", MB_ICONINFORMATION);
		theDoc.DeleteContents();
		return FALSE;
	}
	// Once open, map the file to a CArchive object for the document to use.
	try
	{
		pArchive = new CArchive(pFile, CArchive::load);
	}
	catch (CException *e)
	{
		e->Delete();
		theDoc.DeleteContents();
		MessageBox("CArchive creation failed", "FlashQuiz", MB_ICONINFORMATION);
		pFile->Close();
		delete pFile;
		return FALSE;
	}
	// Tell the document to load itself from the archive.
	theDoc.Serialize(*pArchive);
	if (theDoc.m_bLoadFail)
	{
		MessageBox("File load failed", "FlashQuiz", MB_ICONINFORMATION);
		theDoc.DeleteContents();
		theDoc.Set_File_Name(NULL);
	}
	else
	{
		theDoc.Set_File_Name(pszFileName);
		m_cmdTakeQuiz.EnableWindow(theDoc.m_intQuestionCount != 0);
		m_cmdEdit.EnableWindow(TRUE);
		Set_Doc_Title();
	}
	pArchive->Close();	// Close the archive.
	pFile->Close();		// Close the file.
	delete pArchive;	// Delete the archive & file objects.
	delete pFile;
	return (!theDoc.m_bLoadFail);	// Indicate success or failure.
}

// If appropriate, appends results to quiz log.
void CFlashQuizDlg::Do_Log()
{
	CLogScore dlg;

	if (theDoc.m_bLog)								// Log flag set?
		if (!theDoc.m_strLogFileName.IsEmpty())		// Log filename set?
			if (dlg.DoModal() == IDOK)				// Ask user for comment.
				theDoc.Log_Quiz(dlg.m_strComment);	// If he says OK, write the log entry.
}
