// MyPicture.cpp : implementation file
//
// FlashQuiz v1.0
// Copyright  2000 Ziff Davis Media, Inc.
// Written by Gregory A. Wolking.

#include "stdafx.h"
#include "flashquiz.h"
#include "MyPicture.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMyPicture

CMyPicture::CMyPicture()
{
}

CMyPicture::~CMyPicture()
{
}


BEGIN_MESSAGE_MAP(CMyPicture, CStatic)
	//{{AFX_MSG_MAP(CMyPicture)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMyPicture message handlers


///////////////////////////////////////////
// CMyPicture member functions


// Rescales the specified bitmap to fit the current client area while maintaining
// the image's aspect ratio.

// If bDeleteOriginal is TRUE, the specified bitmap will be deleted
// if the function succeeds. Otherwise, it will be left unchanged.

// For this function to work properly, the associated static control
// must have the SS_BITMAP and SS_REALSIZEIMAGE styles, which may
// also be combined with the SS_CENTERIMAGE style.
// If SS_CENTERIMAGE is not specified, the control will automatically
// resize itself to fit the scaled image while retaining its current width
// or height, whichever is necessary to retain the image's aspect ratio.
// Otherwise, the image will be centered (horizontally or vertically, as
// required by the aspect ratio) within the control, and the control's
// excess client area will be filled with the same as the pixel at the
// top-left corner of the scaled image.
BOOL CMyPicture::Set_Scaled_Bitmap(HBITMAP hBitmap, BOOL bDeleteOriginal)
{
	int			iHeight, iWidth;
	float		fRatio;
	CRect		rect;
	BITMAP		bmp;
	HBITMAP		hOldBitmap = NULL,
				hNewBitmap = NULL;
	CBitmap		*pBitmap;

	if (hBitmap == NULL)
		return FALSE;
	GetClientRect(&rect);	// Get size of client area.
	// Get pointer to a temporary CBitmap attached to the bitmap.
	pBitmap = CBitmap::FromHandle(hBitmap);
	// Retreive bitmap information.
	pBitmap->GetBitmap(&bmp);
	// Get aspect ratio of the image.
	fRatio = (float) bmp.bmWidth / bmp.bmHeight;
	// Calculate scaled height based on client area width.
	iHeight = (int) (rect.right / fRatio);
	// Calculate scaled width based on client area height.
	iWidth = (int) (rect.bottom * fRatio);
	if (iHeight <= rect.bottom)	// If scaled height will fit,
		rect.bottom = iHeight;	// use it.
	else						// Otherwise,
		rect.right = iWidth;	// use scaled width.
	// Copy and scale the bitmap to a new bitmap of the desired size,
	// leaving the original bitmap intact.
	hNewBitmap = (HBITMAP) CopyImage(hBitmap, IMAGE_BITMAP, rect.right, rect.bottom, bDeleteOriginal ? LR_COPYDELETEORG : 0);
	if (hNewBitmap == NULL)
	{
		return FALSE;
	}
	else
	{
		hOldBitmap = SetBitmap(hNewBitmap);	// Set new bitmap, get handle to old bitmap, if any.
		if (hOldBitmap != NULL)				// If there was an old bitmap,
			DeleteObject(hOldBitmap);			// be sure to destroy it.
		return TRUE;	// Signal success to caller.
	}
}

// This overload uses a bitmap loaded from the specified file.
// The original bitmap will be deleted after it is loaded;
// a copy of the bitmap will be attached to the image control.
BOOL CMyPicture::Set_Scaled_Bitmap(CString strFileName)
{
	HBITMAP		hBitmap = NULL;

	hBitmap = (HBITMAP) LoadImage(NULL, strFileName, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
	if (hBitmap == NULL)	// If image load failed,
	{
		return FALSE;		// return result to caller.
	}
	else
	{
		if (Set_Scaled_Bitmap(hBitmap, TRUE))
		{
			return TRUE;
		}
		else
		{
			DeleteObject(hBitmap);
			return FALSE;
		}
	}
}