// QuizEditor.cpp : implementation file
//
// FlashQuiz v1.0
// Copyright  2000 Ziff Davis Media, Inc.
// Written by Gregory A. Wolking.

#include "stdafx.h"
#include "FlashQuiz.h"
#include "ParmDlg.h"
#include "QEdit.h"
#include "FCEdit.h"
#include "QuizEditor.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CFlashQuizApp theApp;
extern CFlashQuizDoc theDoc;

const char pszNoText[] = "<NO TEXT ENTERED>";

/////////////////////////////////////////////////////////////////////////////
// CQuizEditor dialog


CQuizEditor::CQuizEditor(CWnd* pParent /*=NULL*/)
	: CDialog(CQuizEditor::IDD, pParent)
{
	//{{AFX_DATA_INIT(CQuizEditor)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CQuizEditor::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CQuizEditor)
	DDX_Control(pDX, IDC_cmdSaveAs, m_cmdSaveAs);
	DDX_Control(pDX, IDC_cmdSave, m_cmdSave);
	DDX_Control(pDX, IDC_cmdInsert, m_cmdInsert);
	DDX_Control(pDX, IDC_cmdMove2, m_cmdMove2);
	DDX_Control(pDX, IDC_cmdMove, m_cmdMove);
	DDX_Control(pDX, IDC_lstQuestions, m_lstQuestions);
	DDX_Control(pDX, IDC_cmdEditQuestion, m_cmdEditQuestion);
	DDX_Control(pDX, IDC_cmdDeleteQuestion, m_cmdDeleteQuestion);
	DDX_Control(pDX, IDC_cmdAddQuestion, m_cmdAddQuestion);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CQuizEditor, CDialog)
	//{{AFX_MSG_MAP(CQuizEditor)
	ON_BN_CLICKED(IDC_cmdAddQuestion, OncmdAddQuestion)
	ON_BN_CLICKED(IDC_cmdDeleteQuestion, OncmdDeleteQuestion)
	ON_BN_CLICKED(IDC_cmdEditQuestion, OncmdEditQuestion)
	ON_LBN_DBLCLK(IDC_lstQuestions, OnDblclklstQuestions)
	ON_LBN_SELCHANGE(IDC_lstQuestions, OnSelchangelstQuestions)
	ON_LBN_SETFOCUS(IDC_lstQuestions, OnSetfocuslstQuestions)
	ON_BN_CLICKED(IDC_cmdInsert, OncmdInsert)
	ON_BN_CLICKED(IDC_cmdParameters, OncmdParameters)
	ON_BN_CLICKED(IDC_cmdMove, OncmdMove)
	ON_BN_CLICKED(IDC_cmdMove2, OncmdMove2)
	ON_BN_CLICKED(IDC_cmdSave, OncmdSave)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_cmdSaveAs, OncmdSaveAs)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CQuizEditor message handlers

void CQuizEditor::OncmdAddQuestion() 
{
	int			iNewQuestion,
				iIndex;
	BOOL		bQuizType;
	CQEdit		dlgMultChoice;
	CFCEdit		dlgFlashCard;
	CString		strTemp;
	CQuestion	*pQuestion;

	bQuizType = (theDoc.m_intQuizType == 0);
	if (bQuizType)
	{
		dlgMultChoice.m_bMode = FALSE;
		dlgMultChoice.m_intItem = m_lstQuestions.GetCount() + 1;
	}
	else
	{
		dlgFlashCard.m_bMode = FALSE;
		dlgFlashCard.m_intItem = m_lstQuestions.GetCount() + 1;
	}

	while ((bQuizType ? dlgMultChoice.DoModal() : dlgFlashCard.DoModal()) == IDOK)
	{
		pQuestion = (bQuizType ? dlgMultChoice.m_pQuestion : dlgFlashCard.m_pQuestion);
		if (pQuestion != NULL)
		{
			iNewQuestion = theDoc.Add_Question(pQuestion);
			if (iNewQuestion >= 0)
			{
				if (pQuestion->m_strQuestion.IsEmpty())
					strTemp.Format("%u. %s", iNewQuestion + 1, pszNoText);
				else
					strTemp.Format("%u. %s", iNewQuestion + 1, pQuestion->m_strQuestion);
				iIndex = m_lstQuestions.AddString(strTemp);
				m_lstQuestions.SetCurSel(iIndex);
				if (theDoc.m_intQuestionCount >= 512)
				{
					MessageBox("This quiz is now full.", "FlashQuiz", MB_ICONINFORMATION);
					break;
				}
			}
			if (bQuizType)
			{
				dlgMultChoice.m_pQuestion = NULL;
				dlgMultChoice.m_intItem = m_lstQuestions.GetCount() + 1;
			}
			else
			{
				dlgFlashCard.m_pQuestion = NULL;
				dlgFlashCard.m_intItem = m_lstQuestions.GetCount() + 1;
			}
		}
		else
			break;
	}
	Set_Button_Status();
}

void CQuizEditor::OncmdDeleteQuestion() 
{
	int			i, 
				iPos;
	CString		strTemp;
	CQuestion	*pQuestion;

	if (m_lstQuestions.GetCount() == 0)
		return;
	iPos = m_lstQuestions.GetCurSel();
	if (iPos == LB_ERR)
		return;
	strTemp.Format("Are you sure you wish to delete %s %u?",
		(theDoc.m_intQuizType == 0) ? "question" : "flash card",
		iPos + 1);
	if (MessageBox(strTemp, "Confirm Deletion", MB_ICONQUESTION | MB_YESNO) != IDYES)
		return;
	theDoc.Delete_Question(iPos);
	m_lstQuestions.DeleteString(iPos);
	for (i = iPos; i < theDoc.m_intQuestionCount; i++)
	{
		pQuestion = theDoc.Get_Question(i);
		if (pQuestion->m_strQuestion.IsEmpty())
			strTemp.Format("%u. %s", i + 1, pszNoText);
		else
			strTemp.Format("%u. %s", i + 1, pQuestion->m_strQuestion);
		m_lstQuestions.DeleteString(i);
		m_lstQuestions.InsertString(i, strTemp);
	}
	if (theDoc.m_intQuestionCount != 0)
	{
		if (iPos < theDoc.m_intQuestionCount)
			m_lstQuestions.SetCurSel(iPos);
		else
			m_lstQuestions.SetCurSel(theDoc.m_intQuestionCount - 1);
	}
	Set_Button_Status();
}

void CQuizEditor::OncmdEditQuestion() 
{
	int		iIndex, iResult;
	CQEdit	dlgMultChoice;
	CFCEdit	dlgFlashCard;
	CString	strTemp;
	CQuestion* pQuestion;

	iIndex = m_lstQuestions.GetCurSel();
	if (iIndex == LB_ERR)
		return;
	pQuestion = theDoc.Get_Question(iIndex);
	if (theDoc.m_intQuizType == 0)
	{
		dlgMultChoice.m_pQuestion = pQuestion;
		dlgMultChoice.m_bMode=TRUE;
		dlgMultChoice.m_intItem = iIndex + 1;
		iResult = dlgMultChoice.DoModal();
	}
	else
	{
		dlgFlashCard.m_pQuestion = pQuestion;
		dlgFlashCard.m_bMode=TRUE;
		dlgFlashCard.m_intItem = iIndex + 1;
		iResult = dlgFlashCard.DoModal();
	}
	if (iResult == IDOK)
	{
		if (pQuestion->m_strQuestion.IsEmpty())
			strTemp.Format("%u. %s", iIndex + 1, pszNoText);
		else
			strTemp.Format("%u. %s", iIndex + 1, pQuestion->m_strQuestion);
		m_lstQuestions.DeleteString(iIndex);
		m_lstQuestions.InsertString(iIndex, strTemp);
		m_lstQuestions.SetCurSel(iIndex);
		Set_Button_Status();
	}
}

void CQuizEditor::OnDblclklstQuestions() 
{
	OncmdEditQuestion();
}

void CQuizEditor::OnSelchangelstQuestions() 
{
	Set_Button_Status();
}

void CQuizEditor::OnSetfocuslstQuestions() 
{
	Set_Button_Status();
}

BOOL CQuizEditor::OnInitDialog() 
{
	int			i;
	CString		strTemp;
	CQuestion	*pQuestion;

	CDialog::OnInitDialog();
	if (theDoc.m_intQuizType == 1)
	{
		m_cmdAddQuestion.SetWindowText("&Append flash cards");
		m_cmdInsert.SetWindowText("&Insert flash cards");
		m_cmdEditQuestion.SetWindowText("&Edit flash card");
		m_cmdDeleteQuestion.SetWindowText("&Delete flash card");
		m_cmdMove.SetWindowText("Move flash card &up");
		m_cmdMove2.SetWindowText("&Move flash card down");
	}
	for (i = 0; i < theDoc.m_intQuestionCount; i++)
	{
		pQuestion = theDoc.Get_Question(i);
		if (pQuestion != NULL)
		{
			if (pQuestion->m_strQuestion.IsEmpty())
				strTemp.Format("%u. %s", i + 1, pszNoText);
			else
				strTemp.Format("%u. %s", i + 1, pQuestion->m_strQuestion);
			m_lstQuestions.AddString(strTemp);
		}
	}
	if (theDoc.m_intQuestionCount == 0)
	{
		m_cmdDeleteQuestion.EnableWindow(FALSE);
		m_cmdEditQuestion.EnableWindow(FALSE);
	}
	else
	{
		m_lstQuestions.SetCurSel(0);
	}
	Set_Button_Status();
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CQuizEditor::OncmdInsert() 
{
	int			iPos,
				iCount;
	BOOL		bQuizType;
	CQEdit		dlgMultChoice;
	CFCEdit		dlgFlashCard;
	CString		strTemp;

	if (m_lstQuestions.GetCount() > 0)
		iPos = m_lstQuestions.GetCurSel();
	else
		iPos = 0;
	bQuizType = theDoc.m_intQuizType == 0;
	if (bQuizType)
	{
		dlgMultChoice.m_bMode = FALSE;
		dlgMultChoice.m_intItem = iPos + 1;
	}
	else
	{
		dlgFlashCard.m_bMode = FALSE;
		dlgFlashCard.m_intItem = iPos + 1;
	}
	while ((bQuizType ? dlgMultChoice.DoModal() : dlgFlashCard.DoModal()) == IDOK)
	{
		if ((bQuizType ? dlgMultChoice.m_pQuestion : dlgFlashCard.m_pQuestion) != NULL)
		{
			if (bQuizType ?	theDoc.Insert_Question(iPos, dlgMultChoice.m_pQuestion) : theDoc.Insert_Question(iPos, dlgFlashCard.m_pQuestion))
			{
				strTemp.Format("%u. %s", iPos + 1, (bQuizType ? dlgMultChoice.m_pQuestion->m_strQuestion : dlgFlashCard.m_pQuestion->m_strQuestion));
				m_lstQuestions.InsertString(iPos, strTemp);
				m_lstQuestions.SetCurSel(iPos);
				iCount = m_lstQuestions.GetCount();
				++iPos;
				if (iPos < iCount)
				{
					Rebuild_List(iPos);
				}
				if (theDoc.m_intQuestionCount >= 512)
				{
					MessageBox("This quiz is now full.", "FlashQuiz", MB_ICONINFORMATION);
					break;
				}
			}
			if (bQuizType)
			{
				dlgMultChoice.m_pQuestion = NULL;
				++dlgMultChoice.m_intItem;
			}
			else
			{
				dlgFlashCard.m_pQuestion = NULL;
				++dlgFlashCard.m_intItem;
			}
		}
		else
			break;
	}
	Set_Button_Status();
}

void CQuizEditor::OncmdParameters() 
{
	CParmDlg dlg;
	dlg.m_bInitialParms = FALSE;
	dlg.DoModal();
	Set_Button_Status();
}

void CQuizEditor::OncmdMove() 
{
	int iItem;

	iItem = m_lstQuestions.GetCurSel();
	if (theDoc.Move_Item(TRUE, iItem))
	{
		Rebuild_List(iItem - 1);
		m_lstQuestions.SetCurSel(iItem - 1);
		Set_Button_Status();
	}
}

void CQuizEditor::OncmdMove2() 
{
	int iItem;

	iItem = m_lstQuestions.GetCurSel();
	if (theDoc.Move_Item(FALSE, iItem))
	{
		Rebuild_List(iItem);
		m_lstQuestions.SetCurSel(iItem + 1);
		Set_Button_Status();
	}
}

void CQuizEditor::Set_Button_Status()
{
	int		iSel, iCount;
	BOOL	bStatus;

	Set_Doc_Title();
	iSel = m_lstQuestions.GetCurSel();
	iCount = m_lstQuestions.GetCount();
	// Status = list box is not empty and has an item selected.
	bStatus = (iCount != 0 && iSel != LB_ERR);
	m_cmdEditQuestion.EnableWindow(bStatus);
	m_cmdDeleteQuestion.EnableWindow(bStatus);
	// Move up enabled if list is not empty, contains more than one item, and first item is not selected.
	m_cmdMove.EnableWindow(bStatus && (iCount != 1) && (iSel > 0));
	// Move down enabled if list is not empty, contains more than one item, and last item is not selected.
	m_cmdMove2.EnableWindow(bStatus && (iCount != 1) && (iSel < (iCount - 1)));
	m_cmdSave.EnableWindow(theDoc.IsModified() && iCount > 0);
	m_cmdSaveAs.EnableWindow(iCount > 0);
	bStatus = theDoc.m_intQuestionCount < 512;
	m_cmdAddQuestion.EnableWindow(bStatus);
	m_cmdInsert.EnableWindow(bStatus);
}

void CQuizEditor::Rebuild_List(int iStart)
{
	int			i, iCount;
	CQuestion	*pQuestion;
	CString		strTemp;

	iCount = m_lstQuestions.GetCount();

	for (i = iStart; i < iCount; i++)
	{
		pQuestion = theDoc.Get_Question(i);
		if (pQuestion->m_strQuestion.IsEmpty())
			strTemp.Format("%u. %s", i + 1, pszNoText);
		else
			strTemp.Format("%u. %s", i + 1, pQuestion->m_strQuestion);
		m_lstQuestions.DeleteString(i);
		m_lstQuestions.InsertString(i, strTemp);
	}
}

// Always ask for filename before saving.
void CQuizEditor::OncmdSaveAs() 
{
	if (theDoc.Get_Save_Name(this) == IDOK)
	{
		theDoc.Do_Save(this);
		Set_Doc_Title();
		Set_Button_Status();
	}
}

void CQuizEditor::OncmdSave() 
{
	theDoc.Do_Save(this); // Queries for filename only if not already set.
	Set_Button_Status();
}

void CQuizEditor::OnOK() 
{
	if (theDoc.Do_Query_Save(this))
		CDialog::OnOK();
}

void CQuizEditor::OnClose() 
{
	if (theDoc.Do_Query_Save(this))
		CDialog::OnClose();
}

void CQuizEditor::Set_Doc_Title()
{
	int		iPos;
	CString strTemp,
			strText("Editing New Quiz");

	if (!theDoc.m_strFileName.IsEmpty())
	{
		iPos = theDoc.m_strFileName.ReverseFind('\\');
		if (iPos >= 0)
		{
			strTemp = theDoc.m_strFileName.Mid(iPos + 1);
			strText.Format("Editing %s", strTemp);
		}
	}
	SetWindowText(strText);
}
